"""
carnet.py  Version 10 (finale)
Gestion de fiches de contacts avec
prenom, nom, naissance, courriel (email)
Brendan Scott 2022 VF par OE
"""

### Section Imports
import pickle as saumure
import os.path             #AJOUT

### Section Constantes (ajout de 2 constantes)
NOMFIC_SAUVE   = "carnet.pickle"
QUIT_CONFIRMER = "Vous confirmez vouloir quitter (O/N)? "
INSTRUCTIONS = """
*********************************************
       Application Carnet d'Adresses
(Python en s'amusant Pour les Nuls, Projet 9)
*********************************************
Tapez une des quatre touches suivantes :
A pour Ajouter une personne
L pour la Liste des fiches du carnet
I pour revoir ces Instructions
Q pour Quitter."""
PATRON_LISTE = "%s %s Naissance: %s Email: %s"

### Section Classes
class CarnetAdr(object):
    """ Conteneur de fiches
    METHODES : __init__,ajouter_fiche(), sauver()
    """
    def __init__(self):
        """ Cree l'attribut gens comme liste vide.
        """
        self.gens = []

    def ajouter_fiche(self, fiche_nouvo):
        """ Ajoute une fiche dans gens.
        """
        self.gens.append(fiche_nouvo)

    def sauver(self):
        with open(NOMFIC_SAUVE,'wb') as ficow:
            saumure.dump(self, ficow)

class FicheAdr(object):
    """ Fiche d'un contact.
    METHODES : __init__, __repr__
    """
    def __init__(self, prenom=None, nomfami=None,
                 datenaiss=None, courriel=None):
        """Initialise les 4 attributs.
        """
        self.prenom    = prenom
        self.nomfami   = nomfami
        self.datenaiss = datenaiss
        self.courriel  = courriel

    def __repr__(self):
        """ Produit une chaine selon l'objet
        FicheAdr fourni en argument self.
        """
        patron = "FicheAdr(prenom ='%s', "+\
             "nomfami ='%s',"+\
             "datenaiss ='%s', "+\
             "courriel ='%s')"
        return patron%(self.prenom,self.nomfami,
                       self.datenaiss,self.courriel)

class Kontroleur(object):
    """ Pour gerer les donnees stockees dans une
    instance de CarnetAdr, le menu et le chargement
    des donnees du fichier.
    METHODES : __init__, charger(), gerer_menu(),
    ajouter_fiche(), lister_fiches()
    """
    def __init__(self):
        """
        Initialise le controleur. Cherche un fichier.
        S'il le trouve, il le lit. Sinon, il cree un
        fichier vide.
        """
        self.carnet_adr = self.charger()
        if self.carnet_adr is None:
            self.carnet_adr = CarnetAdr()

        self.gerer_menu()                #AJOUT

    def charger(self):
        """
        Charge un carnet depuis un fichier pickle.
        """
        if os.path.exists(NOMFIC_SAUVE):            #AJOUT
            with open(NOMFIC_SAUVE, 'rb') as ficor: #MODIF
                return saumure.load(ficor)          #MODIF
        else:                                       #AJOUT
            return None                             #AJOUT

#Nouvelle methode
    def gerer_menu(self):
        # Boucle centrale de l'application.
        # Obtient la commande clavier et lance l'action
        print(INSTRUCTIONS)
        while True:
            cmd = input("\n(a)jout,(l)iste,(i)nstructions,(q)uitter : ")
            if cmd == "a":
                self.ajouter_fiche()
            elif cmd == "q":
                if confirmer_quitter():
                    print("Sauvegarde")
                    self.carnet_adr.sauver()
                    print("Fin de l'application")
                    break
            elif cmd == "i":
                print(INSTRUCTIONS)
            elif cmd == "l":
                self.lister_fiches()
            else:
                modele = "*** Touche de commande inconnue (%s) !"
                print(modele%cmd)

    def ajouter_fiche(self): #Methode rendue active
        # Demande la saisie des champs de la nouvelle fiche
        print("Ajout d'une fiche dans le carnet")
        print("Description de la personne :")
        prenom = input("Son prenom ? ")
        if prenom == "q":
            print("On abandonne !")
            return

        nomfami = input("Son nom de famille ? ")
        if nomfami == "q":
            print("On abandonne !")
            return

        M_DATENAISS = "Sa date de naissance (An/Mois/Jour) ? "
        datenaiss = input(M_DATENAISS)
        if datenaiss == "q":
            print("On abandonne !")
            return

        courriel = input("Son @dresse ? ")
        if courriel == "q":
            print("On abandonne !")
            return

        nfiche = FicheAdr(prenom, nomfami, datenaiss, courriel)
        self.carnet_adr.ajouter_fiche(nfiche)
        ncontact = (prenom, nomfami)
        print("Nouvelle fiche faite pour %s %s.\n"%ncontact)

    def lister_fiches(self):
        """ Affiche la liste des fiches du carnet """
        print("\nListe des personnes")
        for indice, e in enumerate(self.carnet_adr.gens):
            contact = (e.prenom, e.nomfami,
                e.datenaiss, e.courriel)
            fiche = PATRON_LISTE%contact
            print("%s: %s"%(indice + 1, fiche))
            # Le comptage commence a 1

### Section Fonctions
def confirmer_quitter():
    """ On sort seulement si saisie de la
    lettre n minuscule par renvoi de False. """
    confi = input(QUIT_CONFIRMER)
    if confi == 'n':
        return False
    else:
        return True

### Section principale main
if __name__ == "__main__":
    controleur = Kontroleur()
#    print(controleur.carnet_adr.gens)  #SUPPR V10
